@@ -1,7 +1,6 @@
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, Any, Dict
 
 from .region import Region
-from .typing import Parsed
 
 if TYPE_CHECKING:
     from .document import Document
@@ -9,7 +8,7 @@ if TYPE_CHECKING:
 
 class SybilFailure(AssertionError):
 
-    def __init__(self, example: 'Example', result: str):
+    def __init__(self, example: 'Example', result: str) -> None:
         super(SybilFailure, self).__init__((
             'Example at {}, line {}, column {} did not evaluate as expected:\n'
             '{}'
@@ -18,6 +17,16 @@ class SybilFailure(AssertionError):
         self.result = result
 
 
+class NotEvaluated(Exception):
+    """
+    An exception that can be raised by an :any:`Evaluator` previously
+    :meth:`pushed <sybil.Document.push_evaluator>` onto the document to indicate that
+    it is not evaluating the current example and that a previously pushed evaluator, or the
+    :any:`Region` evaluator if no others have been pushed, should be used to evaluate the
+    :any:`Example` instead.
+    """
+
+
 class Example:
     """
     This represents a particular example from a documentation source file.
@@ -27,8 +36,8 @@ class Example:
     """
 
     def __init__(
-        self, document: 'Document', line: int, column: int, region: Region, namespace: dict
-    ):
+        self, document: 'Document', line: int, column: int, region: Region, namespace: Dict[str, Any]
+    ) -> None:
         #: The :class:`~sybil.document.Document` from which this example came.
         self.document: 'Document' = document
         #: The absolute path of the :class:`~sybil.document.Document`.
@@ -49,10 +58,10 @@ class Example:
         self.end: int = region.end
         #: The version of this example provided by the parser that yielded
         #: the :class:`~sybil.Region` containing it.
-        self.parsed: Parsed = region.parsed
-        #: The :attr:`~sybil.document.Document.namespace` of the document from
+        self.parsed: Any = region.parsed
+        #: The :attr:`~sybil.Document.namespace` of the document from
         #: which this example came.
-        self.namespace: dict = namespace
+        self.namespace: Dict[str, Any] = namespace
 
     def __repr__(self) -> str:
         return '<Example path={} line={} column={} using {!r}>'.format(
@@ -60,7 +69,5 @@ class Example:
         )
 
     def evaluate(self) -> None:
-        evaluator = self.document.evaluator or self.region.evaluator
-        result = evaluator(self)
-        if result:
-            raise SybilFailure(self, result)
+        if self.region.evaluator is not None:
+            self.document.evaluate(self, self.region.evaluator)
